//**********************************************************************
//**********************************************************************
`timescale 1ns / 10 ps
//05-01-2002 Bryce Morgan
//AT24C02 Verilog Model 
//This model was developed to accurately describe the functional behavior
//of the Atmel Serial EEPROM Product AT24C02.

module  AT24C02(
  sda,
  scl,
  wp);
inout sda;						// Serial data
input scl;						// Serial clock
input wp;						     // Write protection
parameter DEVICE      = "AT24C02";      // Device name
parameter Device_Address = 7'b1010000;	// Device Address
parameter MEM_SIZE     = 256 ;           // Number of bytes in memory
parameter PAGE_SIZE    =  8 ;            // Number of bytes in each page
parameter BYTE_SIZE    =  8 ;            // Number of bits in a byte
parameter ADDR_SIZE     = 8 ;            // number of address bits
parameter ADDR_REG_SIZE = 8 ;            // Size of address register 
parameter Time_out_for_Write = 40_000;		// Shorten for test
//parameter Time_out_for_Write = 10_000_000;		// Scaled to 10 ms.
parameter tAA= 550;	  // Worst-case Clock Low to Data Out Valid

//***************************************************************************
//***************************************************************************
//                           Memory Boundaries                             
//***************************************************************************
//***************************************************************************

reg [(ADDR_REG_SIZE - 1):0]  	addr_reg ;    // 8-bit address register.
reg [(BYTE_SIZE - 1):0] 	memory[(MEM_SIZE-1):0] ;    //Main Memory
reg [BYTE_SIZE-1: 0] 		S_Byte_Shft_Reg;

reg load_address_bit, M_ACK;
reg ld_S_Byte_Shft_Reg;
reg ld_addr_reg_LSB_byte;
reg shift_in;
reg shift_out;

parameter 			send = 1, receive = 0;
parameter 	[31:0]	sending = "send";
parameter 	[31:0]	receiving = "recv";
reg 		[2:0] 	state_recv, next_state_recv;
reg 		[2:0] 	state_send, next_state_send;
wire				sda_out;
reg 				S_send_rcvb;
reg 				Addr_Done;
reg S_ACK;
//reg Valid_Address_and_A1_flg;
reg Valid_Device_Address;

//************************************************************************
//************************// Slave bus controls  *************************
//************************************************************************

reg NO_ACK_flag;	// Flag indicating No_ACK from master
reg TOFG;			// Time out flag

tri	sda = (S_send_rcvb == send)? ((S_ACK == 0)? sda_out: 1'b0): 1'bz;
tri 	sda_in = (S_send_rcvb == receive)? sda : 1'bz;
wire	[31:0] receiver = (S_send_rcvb == send)? 
               sending : ((S_send_rcvb == receive)) ? 
                 receiving : 32'bx;   

event S_START_condition;
event S_STOP_condition;
event incr_addr_reg;
reg S_START, S_STOP;
//************************************************************************
//********************* S_START and S_STOP conditions ********************
//************************************************************************

always @ (negedge sda_in) if(scl == 1)begin -> S_START_condition; 
  $display("************************************** START condition detected");
  S_START = 1; S_STOP = 0;
  Valid_Device_Address = 0;
  @ (posedge scl) S_START = 0; 
end

always @ (posedge sda_in) if(scl == 1) begin -> S_STOP_condition;
S_STOP = 1; Valid_Device_Address = 0;
disable MACHINE_LOOP.Main_Loop ; // added by xujun
$display ("*************************************** STOP condition detected");
end


//************************************************************************
//************************ Memory Initialization *************************
//************************************************************************

integer k;
initial for (k = 0; k<= MEM_SIZE-1; k = k+1) begin
memory[k] = 8'hFF;
end

//initial for (k = 0; k<= MEM_SIZE-1; k = k+1) begin    //modified by zxc
//memory[k] = k;
//end

wire [7:0] mem_byte = memory[addr_reg[7:0]]; //@@ Used for debug to watch memory

//************************************************************************
//*************************** Address Register Controls ******************
//************************************************************************

always @ (posedge scl) 
 if (ld_addr_reg_LSB_byte == 1)   
addr_reg[ADDR_REG_SIZE-1:0] <= S_Byte_Shft_Reg[7:0]; 
// else if (ld_addr_reg_LSB_byte == 1) 
//addr_reg[(ADDR_SIZE-1)/2 -1 : 0] <= S_Byte_Shft_Reg;

// Increment after READ  
always @ (incr_addr_reg) addr_reg <= addr_reg +1; 
  

//*************************************************************************
//**************************     Shift Register    ************************
//*************************************************************************

assign sda_out = S_Byte_Shft_Reg [BYTE_SIZE-1];

always @(posedge scl)
if (shift_in == 1) 
     S_Byte_Shft_Reg <= { S_Byte_Shft_Reg [BYTE_SIZE-2:0], sda_in };
else 
   if (shift_out == 1) 
      @(negedge scl) 
        S_Byte_Shft_Reg <= S_Byte_Shft_Reg << 1;   //switching on every positive_edge
   else 
     if (ld_S_Byte_Shft_Reg == 1) S_Byte_Shft_Reg <= memory[addr_reg[7:0]];       

//*************************************************************************
//*********************** Read/Write State Machine ************************
//*************************************************************************

always begin : MACHINE_LOOP
    Power_up_initialization;
    forever begin : Main_Loop
      $display("Begin Main Loop");
      S_send_rcvb = receive;
      $display ("Waiting for START_condition");
      @ (S_START_condition)
         begin :Data_Transmission
           $display("Data_Transmission");
           Get_a_Byte;		 
           Parse_the_Address_Byte_and_Read_or_Write_if_Valid;
         end // Data_Transmission
    end  // Main_Loop
  end  // Machine_Loop 

//*************************************************************************
//*************************************************************************

task Get_a_Byte;
  begin
    NO_ACK_flag = 0;
    shift_in = 1;
    repeat (8)
      begin
      @ (posedge scl); 
      end
  end
endtask

task Parse_the_Address_Byte_and_Read_or_Write_if_Valid;
      begin 
        @ (negedge scl) shift_in = 0;
           NO_ACK_flag = 0;
           Check_for_Valid_Device_Address;
	      if (Valid_Device_Address)
               begin:  Device_Address_Match 
                 $display ("Device Address Match");
                 case (S_Byte_Shft_Reg[0]) 
                    0: Write_or_Dummy_Write_with_Random_Read;
                    1: Current_Address_and_Sequential_Read;
                       default: $display("Invalid device address word received");
                 endcase
               end  //Device_Address_Match 
       end
endtask

task Check_for_Valid_Device_Address;
     begin
        if (S_Byte_Shft_Reg[BYTE_SIZE-1: BYTE_SIZE-7] == Device_Address) Valid_Device_Address = 1;
     end
endtask
task Write_or_Dummy_Write_with_Random_Read;
  begin :Write_Sequence  
       if (wp == 1) 
         begin 
         $display ("Chip is write-protected"); 
         disable Write_Sequence; 
         end
    else begin: Chip_not_write_protected
      $display("***************************************** Begin Write Sequence"); 
      Set_Flags_and_Get_Address_Bytes_and_ACK_to_Master;
      shift_in = 1; $display("Address bytes are loaded");
       @ (posedge scl); 				// MSB is saved
        $display("First clock after address bytes are loaded");
        @ (posedge S_START or posedge scl)
        if (S_START ==1)
           begin
            $display("Dummywrite detected - this is the device address byte"); 
            Get_a_Byte;
            Begin_a_Read_Sequence;
           end

       else begin: Byte_or_Page_Write
         $display("Write sequence");
         Get_Remainder_of_Byte;
         ACK_and_write_a_Byte;
         addr_reg[7:0] <= addr_reg[7:0] +1; $display("Increment the address register");
         Execute_page_Write;
         $display ("*******************************End of page write");
         #Time_out_for_Write TOFG = 0;
       end  // Byte_or_Page_Write
     end // Chip_not_write_protected
  end   // Write_Sequence
endtask

task ACK_and_write_a_Byte;
   begin
       $display ("ACK and write a byte");
      @ (negedge scl) S_ACK = 1; shift_in = 0;
      S_send_rcvb = send;
      @ (posedge scl) memory[addr_reg[7:0]] <= S_Byte_Shft_Reg[7:0];
      @ (negedge scl) S_ACK = 0; shift_in = 1;
      S_send_rcvb = receive;
   end
endtask

task Get_Remainder_of_Byte;
  begin
    repeat (6)
      begin
      @ (posedge scl);  
      end
  end
endtask

task Set_Flags_and_Get_Address_Bytes_and_ACK_to_Master;
    begin         
      S_send_rcvb = send;
      S_ACK = 1; 						// Send ACK to master
      @(negedge scl) 
      S_ACK = 0; shift_in = 1;
      S_send_rcvb = receive;
      //Get_MSB_Byte_and_ACK;
      Get_LSB_Byte_and_ACK;
   end
endtask 

task Begin_a_Read_Sequence;
 begin 
   @ (negedge scl)
   shift_in = 0;
   NO_ACK_flag = 0;
   Check_for_Valid_Device_Address;
    if (Valid_Device_Address)
        begin: Device_Address_Match 
           $display ("Device Address Match");
           case (S_Byte_Shft_Reg[0]) 
           0: Write_or_Dummy_Write_with_Random_Read;  // Should not occur
           1: Read_After_Dummy_Write;	   // For READ after a dummy write
           default: $display("Invalid Read protocol sequence received");
           endcase
        end  // Device_Address_Match
 end
endtask

task Execute_page_Write;
  begin
   while (S_STOP != 1)
       begin: Page_Write
        $display("********************************Page write");
         shift_in = 1;
         @ (posedge scl);
         @ (negedge scl)
           if (S_STOP ==1)
             begin
              TOFG = 1;
              shift_in = 0;
              $display("STOP detected - and begin time out for writing"); 
             end
           else
             begin
               repeat (7) @ (posedge scl);
               ACK_and_write_a_Byte;
               addr_reg[7:0] <= addr_reg[7:0] +1;     
               $display("Increment the address register LSB byte");
             end
       end // Page_Write
   end
endtask

task Current_Address_and_Sequential_Read;
   begin
      $display("Beginning Current_Address_and_Sequential_Read");
      Load_Slave_Shift_Register_and_Send_ACK;
      shift_out = 1; 
      @(negedge scl); 
      forever begin :Current_or_Sequential_Read
          repeat (7) @ (negedge scl); // Have sent ACK and 8 bits to master
          shift_out = 0;
          ld_S_Byte_Shft_Reg = 1; 
           S_send_rcvb = receive;
          ->incr_addr_reg;
          @ (posedge scl)
          if (sda_in > 0) 
             begin :Got_NO_ACK_from_Master
               NO_ACK_so_wait_for_STOP_condition;
               disable MACHINE_LOOP.Main_Loop; 
               end //Got_No_ACK_from_Master
          else 
              begin: Got_ACK_from_Master  //  bit was 0
                 Got_ACK_so_prepare_to_send;
              end // Got_ACK_from_Master
      end // Current _or_Sequential_Read 
  end
endtask

task Read_After_Dummy_Write;
   begin
     $display("Reading After Dummy Write");
     Load_Slave_Shift_Register_and_Send_ACK;
         forever 
          begin :Current_or_Sequential_Read
             shift_out = 1; S_send_rcvb = send;
             repeat (8) @ (posedge scl);   // have sent ACK and 8 bits to master
             @ (negedge scl)shift_out = 0;
             ld_S_Byte_Shft_Reg = 1; 
             S_send_rcvb = receive;
             ->incr_addr_reg;
             @ (posedge scl)
             if (sda_in > 0)
                 begin :Got_NO_ACK_from_Master
                    NO_ACK_so_wait_for_STOP_condition;
                    disable MACHINE_LOOP.Main_Loop; 
                 end //Got_No_ACK_from_Master

        else
           begin: Got_ACK_from_Master  //  bit was 0
             $display ("got ACK from master");
              @ (negedge scl);
              ld_S_Byte_Shft_Reg = 0;
              S_send_rcvb = send;
              shift_out = 1;
            end // Got_ACK_from_Master
        end // Current _or_Sequential_Read 
    end
endtask

task Load_Slave_Shift_Register_and_Send_ACK;
    begin
      ld_S_Byte_Shft_Reg = 1;
      S_send_rcvb = send;
      S_ACK = 1;                             // Send ACK to master
      @(negedge scl) 
      ld_S_Byte_Shft_Reg = 0;
       S_ACK = 0;
    end
endtask

task Got_ACK_so_prepare_to_send;
   begin
     $display ("got ACK from master");
     @ (negedge scl);
     ld_S_Byte_Shft_Reg = 0;
     S_send_rcvb = send;
     shift_out = 1;
     @ (negedge scl) ;
   end
endtask

task NO_ACK_so_wait_for_STOP_condition;
   begin
     NO_ACK_flag = 1; $display ("Got_NO_ACK_from_Master ");
     ld_S_Byte_Shft_Reg = 0;
     @ (S_STOP_condition)
       begin // 
         $display ("disable Main_Loop"); 
         NO_ACK_flag = 0; 
       end
   end
endtask                   

//task Get_MSB_Byte_and_ACK;
//  begin
//    Get_a_Byte;
//    @ (negedge scl) ;
//    shift_in = 0;
//    ld_addr_reg_byte = 1;
//    S_send_rcvb = send;
//    S_ACK = 1; 				// Send ACK to master
//    @ (negedge scl) 
//    $display                      ("Loading MSB address byte");
//    ld_addr_reg_byte = 0; 
//    S_send_rcvb = receive;
//    S_ACK = 0;
//  end
//endtask


task Get_LSB_Byte_and_ACK;
   begin
      shift_in = 1;
      Get_a_Byte;
      @ (negedge scl) ;
      shift_in = 0;
      ld_addr_reg_LSB_byte = 1;
      S_send_rcvb = send;
      S_ACK = 1;                      // Send ACK to master
      @ (negedge scl) 
      $display                       ("Loading LSB address byte");
      ld_addr_reg_LSB_byte = 0; 
      S_send_rcvb = receive;
      S_ACK = 0; 
   end
endtask

task Power_up_initialization;		
  begin
    #1 $display ("Power_up_Initialization");
    addr_reg = 0;
    S_send_rcvb = receive;
    ld_S_Byte_Shft_Reg = 0;    
    ld_addr_reg_LSB_byte = 0;
    shift_in = 0;
    shift_out = 0;
    S_ACK = 0;
    NO_ACK_flag = 0;
    TOFG = 0;
    S_START = 0;
    S_STOP = 0;
  end
endtask


//***************************************************************************
//***************************************************************************
//                           Timing Parameters and Checks                             
//***************************************************************************
//***************************************************************************

/*     __________________________________________________                    *  
*     | Symbol  |  Voltage   |  Min   |  Max  |  Units |                    *
*     |---------|------------|--------|-------|--------|                    *
*     |  fscl   |  4.5 - 5.5 |   0    |  400  |   kHz  |                    *
*     |         |  2.7 - 5.5 |   0    |  100  |        |                    *
*     |---------|------------|--------|-------|--------|                    *
*     |  tLOW   |  4.5 - 5.5 |   0.6  |   0   |   us   |                    *
*     |         |  2.7 - 5.5 |   1.3  |   0   |        |                    *
*     |---------|------------|--------|-------|--------|                    *
*     |  tHIGH  |  4.5 - 5.5 |   0.4  |   0   |   us   |                    *
*     |         |  2.7 - 5.5 |   1.0  |   0   |        |                    *
*     |---------|------------|--------|-------|--------|                    *
*     |  tAA    |  4.5 - 5.5 |   0.05 |  0.05 |   us   |                    *
*     |         |  2.7 - 5.5 |   0.05 |  0.9  |        |                    *
*     |---------|------------|--------|-------|--------|                    *
*     | tHDSTA  |  4.5 - 5.5 |   0.25 |       |   us   |                    *
*     |         |  2.7 - 5.5 |   0.6  |       |        |                    *
*     |---------|------------|--------|-------|--------|                    *
*     | tSUSTA  |  4.5 - 5.5 |   0.25 |       |   us   |                    *
*     |         |  2.7 - 5.5 |   0.6  |       |        |                    *
*     |---------|------------|--------|-------|--------|                    *
*     | tHDDAT  |            |        |   0   |   ns   |                    *
*     |---------|------------|--------|-------|--------|                    *
*     | tSUDAT  |            |        |   100 |   ns   |                    *
*     |---------|------------|--------|-------|--------|                    *
*     |  tSUSTO |  4.5 - 5.5 |   0.25 |       |   us   |                    *
*     |         |  2.7 - 5.5 |   0.6  |       |        |                    *
*     |---------|------------|--------|-------|--------|                    *
*     |  tDH    |            |   50   |       |   ns   |                    *
*      ------------------------------------------------                     *
*                                                                           *
****************************************************************************/


specify             
  specparam tLOW    = 600;     
  specparam tHIGH   = 400;
  //specparam tAA     = 50;
  specparam tHDSTA  = 250;
  specparam tSUSTA  = 250;
  specparam tHDDAT  = 0;
  specparam tSUDAT  = 100;
  specparam tSYSTO  = 300;
  specparam tSU     = 250;
  specparam tDH     = 50;

//  $width(posedge scl, tHIGH);            		// scl pulsewidths
//  $width(negedge scl, tLOW);  
  
//  $hold(scl, negedge sda_in, tHDSTA);		// START hold time  
//  $setup(scl, negedge sda_in, tSUSTA);		// START setup time  

//  $hold(sda_in, negedge scl, tHDDAT);		// Data in hold time  
//  $setup(sda_in, posedge scl, tSUDAT);		// Data_in setup time  

// $setup(scl, posedge S_STOP, tSUSTA);		// STOP setup time  

endspecify

endmodule


